"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IndyVdrIndyDidRegistrar = void 0;
const core_1 = require("@aries-framework/core");
const indy_vdr_shared_1 = require("@hyperledger/indy-vdr-shared");
const error_1 = require("../error");
const IndyVdrPoolService_1 = require("../pool/IndyVdrPoolService");
const didIndyUtil_1 = require("./didIndyUtil");
const didSovUtil_1 = require("./didSovUtil");
class IndyVdrIndyDidRegistrar {
    constructor() {
        this.supportedMethods = ['indy'];
    }
    async create(agentContext, options) {
        var _a, _b;
        const seed = (_a = options.secret) === null || _a === void 0 ? void 0 : _a.seed;
        if (seed && (typeof seed !== 'string' || seed.length !== 32)) {
            return {
                didDocumentMetadata: {},
                didRegistrationMetadata: {},
                didState: {
                    state: 'failed',
                    reason: 'Invalid seed provided',
                },
            };
        }
        const { alias, role, submitterDid, submitterVerkey, services, useEndpointAttrib } = options.options;
        let verkey = options.options.verkey;
        let did = options.did;
        let id;
        if (seed && did) {
            return {
                didDocumentMetadata: {},
                didRegistrationMetadata: {},
                didState: {
                    state: 'failed',
                    reason: `Only one of 'seed' and 'did' must be provided`,
                },
            };
        }
        try {
            const { namespace, id: submitterId } = (0, didIndyUtil_1.parseIndyDid)(submitterDid);
            if (did) {
                id = (0, didIndyUtil_1.parseIndyDid)(did).id;
                if (!verkey) {
                    return {
                        didDocumentMetadata: {},
                        didRegistrationMetadata: {},
                        didState: {
                            state: 'failed',
                            reason: 'If a did is defined, a matching verkey must be provided',
                        },
                    };
                }
                if (!(0, didIndyUtil_1.isSelfCertifiedIndyDid)(did, verkey)) {
                    throw new Error(`Initial verkey ${verkey} does not match did ˇ${did}`);
                }
            }
            else {
                // Create a new key and calculate did according to the rules for indy did method
                const key = await agentContext.wallet.createKey({ seed, keyType: core_1.KeyType.Ed25519 });
                const buffer = core_1.Hasher.hash(key.publicKey, 'sha2-256');
                id = core_1.TypedArrayEncoder.toBase58(buffer.slice(0, 16));
                verkey = key.publicKeyBase58;
                did = `did:indy:${namespace}:${id}`;
            }
            // Create base did document
            const didDocumentBuilder = (0, didIndyUtil_1.indyDidDocumentFromDid)(did, verkey);
            let diddocContent;
            // Add services if object was passed
            if (services) {
                services.forEach((item) => didDocumentBuilder.addService(item));
                const commTypes = [core_1.IndyAgentService.type, core_1.DidCommV1Service.type, core_1.DidCommV2Service.type];
                const serviceTypes = new Set(services.map((item) => item.type));
                const keyAgreementId = `${did}#key-agreement-1`;
                // If there is at least a communication service, add the key agreement key
                if (commTypes.some((type) => serviceTypes.has(type))) {
                    didDocumentBuilder
                        .addVerificationMethod({
                        controller: did,
                        id: keyAgreementId,
                        publicKeyBase58: (0, didIndyUtil_1.createKeyAgreementKey)(verkey),
                        type: 'X25519KeyAgreementKey2019',
                    })
                        .addKeyAgreement(keyAgreementId);
                }
                // If there is a DIDComm V2 service, add context
                if (serviceTypes.has(core_1.DidCommV2Service.type)) {
                    didDocumentBuilder.addContext('https://didcomm.org/messaging/contexts/v2');
                }
                if (!useEndpointAttrib) {
                    // create diddocContent parameter based on the diff between the base and the resulting DID Document
                    diddocContent = (0, didIndyUtil_1.didDocDiff)(didDocumentBuilder.build().toJSON(), (0, didIndyUtil_1.indyDidDocumentFromDid)(did, verkey).build().toJSON());
                }
            }
            // Build did document
            const didDocument = didDocumentBuilder.build();
            const pool = agentContext.dependencyManager.resolve(IndyVdrPoolService_1.IndyVdrPoolService).getPoolForNamespace(namespace);
            // If there are services and we are using legacy indy endpoint attrib, make sure they are suitable before registering the DID
            if (services && useEndpointAttrib) {
                const endpoints = (0, didSovUtil_1.endpointsAttribFromServices)(services);
                await this.registerPublicDid(agentContext, pool, submitterId, submitterVerkey, id, verkey, alias, role);
                await this.setEndpointsForDid(agentContext, pool, verkey, id, endpoints);
            }
            else {
                await this.registerPublicDid(agentContext, pool, submitterId, submitterVerkey, id, verkey, alias, role, diddocContent);
            }
            // Save the did so we know we created it and can issue with it
            const didRecord = new core_1.DidRecord({
                did,
                role: core_1.DidDocumentRole.Created,
                tags: {
                    recipientKeyFingerprints: didDocument.recipientKeys.map((key) => key.fingerprint),
                    qualifiedIndyDid: did,
                },
            });
            const didRepository = agentContext.dependencyManager.resolve(core_1.DidRepository);
            await didRepository.save(agentContext, didRecord);
            return {
                didDocumentMetadata: {},
                didRegistrationMetadata: {
                    didIndyNamespace: namespace,
                },
                didState: {
                    state: 'finished',
                    did,
                    didDocument,
                    secret: {
                        // FIXME: the uni-registrar creates the seed in the registrar method
                        // if it doesn't exist so the seed can always be returned. Currently
                        // we can only return it if the seed was passed in by the user. Once
                        // we have a secure method for generating seeds we should use the same
                        // approach
                        seed: (_b = options.secret) === null || _b === void 0 ? void 0 : _b.seed,
                    },
                },
            };
        }
        catch (error) {
            return {
                didDocumentMetadata: {},
                didRegistrationMetadata: {},
                didState: {
                    state: 'failed',
                    reason: `unknownError: ${error.message}`,
                },
            };
        }
    }
    async update() {
        return {
            didDocumentMetadata: {},
            didRegistrationMetadata: {},
            didState: {
                state: 'failed',
                reason: `notImplemented: updating did:indy not implemented yet`,
            },
        };
    }
    async deactivate() {
        return {
            didDocumentMetadata: {},
            didRegistrationMetadata: {},
            didState: {
                state: 'failed',
                reason: `notImplemented: deactivating did:indy not implemented yet`,
            },
        };
    }
    async registerPublicDid(agentContext, pool, submitterDid, submitterVerkey, targetDid, verkey, alias, role, diddocContent) {
        try {
            agentContext.config.logger.debug(`Register public did '${targetDid}' on ledger '${pool}'`);
            // FIXME: Add diddocContent when supported by indy-vdr
            if (diddocContent) {
                throw new error_1.IndyVdrError('diddocContent is not yet supported');
            }
            const request = new indy_vdr_shared_1.NymRequest({ submitterDid, dest: targetDid, verkey, alias });
            const signingKey = core_1.Key.fromPublicKeyBase58(submitterVerkey, core_1.KeyType.Ed25519);
            const response = await pool.submitWriteRequest(agentContext, request, signingKey);
            agentContext.config.logger.debug(`Registered public did '${targetDid}' on ledger '${pool.indyNamespace}'`, {
                response,
            });
            return targetDid;
        }
        catch (error) {
            agentContext.config.logger.error(`Error registering public did '${targetDid}' on ledger '${pool.indyNamespace}'`, {
                error,
                submitterDid,
                targetDid,
                verkey,
                alias,
                role,
                pool: pool.indyNamespace,
            });
            throw error;
        }
    }
    async setEndpointsForDid(agentContext, pool, submitterVerkey, did, endpoints) {
        try {
            agentContext.config.logger.debug(`Set endpoints for did '${did}' on ledger '${pool.indyNamespace}'`, endpoints);
            const request = new indy_vdr_shared_1.AttribRequest({
                submitterDid: did,
                targetDid: did,
                raw: JSON.stringify({ endpoint: endpoints }),
            });
            const signingKey = core_1.Key.fromPublicKeyBase58(submitterVerkey, core_1.KeyType.Ed25519);
            const response = await pool.submitWriteRequest(agentContext, request, signingKey);
            agentContext.config.logger.debug(`Successfully set endpoints for did '${did}' on ledger '${pool.indyNamespace}'`, {
                response,
                endpoints,
            });
        }
        catch (error) {
            agentContext.config.logger.error(`Error setting endpoints for did '${did}' on ledger '${pool.indyNamespace}'`, {
                error,
                did,
                endpoints,
            });
            throw new error_1.IndyVdrError(error);
        }
    }
}
exports.IndyVdrIndyDidRegistrar = IndyVdrIndyDidRegistrar;
//# sourceMappingURL=IndyVdrIndyDidRegistrar.js.map